//
// WTFeedbackView.h    1.0
//
// Created by Wagner Truppel on 2009-10-08.
// Copyright 2009 Wagner Truppel (jrcapab@gmail.com). All rights reserved.
//
// Inspired by and modified from DSActivityView.
//
// =======================================================
// DSActivityView
// Dejal Open Source
//
// Created by David Sinclair on 2009-07-26.
// Copyright 2009 Dejal Systems, LLC. All rights reserved.
// =======================================================
//
// Redistribution and use in binary and source forms, with or without
// modification, are permitted for any project, commercial or otherwise,
// provided that the following conditions are met:
//
// Redistributions in binary form must display the copyright notice in the
// About view, website, and/or documentation.
//
// Redistributions of source code must retain the copyright notice, this
// list of conditions, and the following disclaimer.
//
// THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THIS SOFTWARE.
//

#import <UIKit/UIKit.h>





// ========================================================================= //
                   #pragma mark enums, structs, and typedefs
// ========================================================================= //

typedef enum enWTFeedbackViewKind
{
    WTFeedbackViewKindUnavailable = 0,
    WTFeedbackViewKindActivity,
    WTFeedbackViewKindProgress,
    WTFeedbackViewKindFlexible

} WTFeedbackViewKind;

// ========================================================================= //

typedef enum enWTFeedbackViewStyle
{
    WTFeedbackViewStyleUnavailable = 0,
    WTFeedbackViewStyleSimple,
    WTFeedbackViewStyleBezel,
    WTFeedbackViewStyleKeyboard

} WTFeedbackViewStyle;





// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ //
                                #pragma mark -
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ //

@interface WTFeedbackView : UIView
{
    @protected

        WTFeedbackViewKind         _feedbackKind;
        WTFeedbackViewStyle        _feedbackStyle;

        // Necessary to automatically reposition the
        // feedback label when its text changes.
        CGSize                     _labelTextSize;
        CGRect                     _textBorderFrame;

        UIView*                    _originalView;
        UIView*                    _borderView;
        UILabel*                   _feedbackLabel;

        // These are nil if not applicable to the feedback
        // view being instantiated.
        UIActivityIndicatorView*   _activityIndicatorView;
        UIProgressView*            _progressView;

        BOOL                       _currentlyShowingProgressView;

        // These store the border view frames when the activity
        // indicator view or the progress view are visible,
        // respectively, and are used to animate the switch
        // between the two, for feedback views of kind
        // WTFeedbackViewKindFlexible.
        CGRect                     _activityBorderFrame;
        CGRect                     _progressBorderFrame;
}

// ========================================================================= //
                         #pragma mark factory methods
// ========================================================================= //

// Creates a simple feedback view, containing an activity indicator and
// the feedback label, horizontally next to one another, with the entire
// feedback view centered within the specified view.

// The label text is set to a configurable default value, though it
// can be changed on demand at any time while the feedback view is visible.

// Returns the feedback view, already added as a subview of the specified
// view.

// NOTE: this method returns a retained object. However, there is no need
// for the client code to release it, as that is done when one of the
// dismissal methods is invoked.

+ (WTFeedbackView*) newSimpleActivityViewForView: (UIView*) addToView;

// ========================================================================= //

// Creates a simple feedback view, containing an activity indicator and
// the feedback label, horizontally next to one another, with the entire
// feedback view centered within the specified view.

// The label text is set to the specified string, though it can be changed
// on demand at any time while the feedback view is visible.

// Returns the feedback view, already added as a subview of the specified
// view.

// NOTE: this method returns a retained object. However, there is no need
// for the client code to release it, as that is done when one of the
// dismissal methods is invoked.

+ (WTFeedbackView*) newSimpleActivityViewForView: (UIView*) addToView
                                        withText: (NSString*) labelText;

// ========================================================================= //

// Creates a bezel-style feedback view, containing an activity indicator and
// the feedback label, vertically next to one another, with the entire
// feedback view centered within the specified view.

// The label text is set to a configurable default value, though it
// can be changed on demand at any time while the feedback view is visible.

// Returns the feedback view, already added as a subview of the specified
// view.

// NOTE: this method returns a retained object. However, there is no need
// for the client code to release it, as that is done when one of the
// dismissal methods is invoked.

+ (WTFeedbackView*) newBezelActivityViewForView: (UIView*) addToView;

// ========================================================================= //

// Creates a bezel-style feedback view, containing an activity indicator and
// the feedback label, vertically next to one another, with the entire
// feedback view centered within the specified view.

// The label text is set to the specified string, though it can be changed
// on demand at any time while the feedback view is visible.

// Returns the feedback view, already added as a subview of the specified
// view.

// NOTE: this method returns a retained object. However, there is no need
// for the client code to release it, as that is done when one of the
// dismissal methods is invoked.

+ (WTFeedbackView*) newBezelActivityViewForView: (UIView*) addToView
                                       withText: (NSString*) labelText;

// ========================================================================= //

// Creates a bezel-style feedback view, containing a progress view and
// the feedback label, vertically next to one another, with the entire
// feedback view centered within the specified view.

// The label text is set to a configurable default value, though it
// can be changed on demand at any time while the feedback view is visible.

// Returns the feedback view, already added as a subview of the specified
// view.

// NOTE: this method returns a retained object. However, there is no need
// for the client code to release it, as that is done when one of the
// dismissal methods is invoked.

+ (WTFeedbackView*) newBezelProgressViewForView: (UIView*) addToView;

// ========================================================================= //

// Creates a bezel-style feedback view, containing a progress view and
// the feedback label, vertically next to one another, with the entire
// feedback view centered within the specified view.

// The label text is set to the specified string, though it can be changed
// on demand at any time while the feedback view is visible.

// Returns the feedback view, already added as a subview of the specified
// view.

// NOTE: this method returns a retained object. However, there is no need
// for the client code to release it, as that is done when one of the
// dismissal methods is invoked.

+ (WTFeedbackView*) newBezelProgressViewForView: (UIView*) addToView
                                       withText: (NSString*) labelText;

// ========================================================================= //

// Creates a bezel-style feedback view, containing boh an activity indicator
// and a progress view (though only one can be shown at any given time), and
// the feedback label, vertically next to one another, with the entire
// feedback view centered within the specified view. Which indicator (activity
// or progress) is visible when the feedback view is first displayed is set
// by the specified boolean. The visible indicator can be switched on demand
// at any time, while the feedback view is showing.

// The label text is set to a configurable default value, though it
// can be changed on demand at any time while the feedback view is visible.

// Returns the feedback view, already added as a subview of the specified
// view.

// NOTE: this method returns a retained object. However, there is no need
// for the client code to release it, as that is done when one of the
// dismissal methods is invoked.

+ (WTFeedbackView*) newBezelFlexibleViewForView: (UIView*) addToView
                            showingProgressView: (BOOL) showing;

// ========================================================================= //

// Creates a bezel-style feedback view, containing boh an activity indicator
// and a progress view (though only one can be shown at any given time), and
// the feedback label, vertically next to one another, with the entire
// feedback view centered within the specified view. Which indicator (activity
// or progress) is visible when the feedback view is first displayed is set
// by the specified boolean. The visible indicator can be switched on demand
// at any time, while the feedback view is showing.

// The label text is set to the specified string, though it can be changed
// on demand at any time while the feedback view is visible.

// Returns the feedback view, already added as a subview of the specified
// view.

// NOTE: this method returns a retained object. However, there is no need
// for the client code to release it, as that is done when one of the
// dismissal methods is invoked.

+ (WTFeedbackView*) newBezelFlexibleViewForView: (UIView*) addToView
                            showingProgressView: (BOOL) showing
                                       withText: (NSString*) labelText;

// ========================================================================= //

// Creates a keyboard-style feedback view, containing an activity
// indicator and the feedback label, vertically next to one another,
// with the entire feedback view centered within the keyboard view,
// if the keyboard is showing.

// The label text is set to a configurable default value, though it
// can be changed on demand at any time while the feedback view is visible.

// Returns the feedback view, already added as a subview of the keyboard
// view, or nil if the keyboard isn't showing.

// NOTE: this method returns a retained object. However, there is no need
// for the client code to release it, as that is done when one of the
// dismissal methods is invoked.

+ (WTFeedbackView*) newKeyboardActivityView;

// ========================================================================= //

// Creates a keyboard-style feedback view, containing an activity
// indicator and the feedback label, vertically next to one another,
// with the entire feedback view centered within the keyboard view,
// if the keyboard is showing.

// The label text is set to the specified string, though it can be changed
// on demand at any time while the feedback view is visible.

// Returns the feedback view, already added as a subview of the keyboard
// view, or nil if the keyboard isn't showing.

// NOTE: this method returns a retained object. However, there is no need
// for the client code to release it, as that is done when one of the
// dismissal methods is invoked.

+ (WTFeedbackView*) newKeyboardActivityViewWithText: (NSString*) labelText;

// ========================================================================= //

// Creates a keyboard-style feedback view, containing a progress
// view and the feedback label, vertically next to one another,
// with the entire feedback view centered within the keyboard view,
// if the keyboard is showing.

// The label text is set to a configurable default value, though it
// can be changed on demand at any time while the feedback view is visible.

// Returns the feedback view, already added as a subview of the keyboard
// view, or nil if the keyboard isn't showing.

// NOTE: this method returns a retained object. However, there is no need
// for the client code to release it, as that is done when one of the
// dismissal methods is invoked.

+ (WTFeedbackView*) newKeyboardProgressView;

// ========================================================================= //

// Creates a keyboard-style feedback view, containing a progress
// view and the feedback label, vertically next to one another,
// with the entire feedback view centered within the keyboard view,
// if the keyboard is showing.

// The label text is set to the specified string, though it can be changed
// on demand at any time while the feedback view is visible.

// Returns the feedback view, already added as a subview of the keyboard
// view, or nil if the keyboard isn't showing.

// NOTE: this method returns a retained object. However, there is no need
// for the client code to release it, as that is done when one of the
// dismissal methods is invoked.

+ (WTFeedbackView*) newKeyboardProgressViewWithText: (NSString*) labelText;

// ========================================================================= //

// Creates a keyboard-style feedback view, containing boh an activity indicator
// and a progress view (though only one can be shown at any given time), and
// the feedback label, vertically next to one another, with the entire
// feedback view centered within the keyboard view, if the keyboard is showing.
// Which indicator (activity or progress) is visible when the feedback view is
// first displayed is set by the specified boolean. The visible indicator can
// be switched on demand at any time, while the feedback view is showing.

// The label text is set to a configurable default value, though it
// can be changed on demand at any time while the feedback view is visible.

// Returns the feedback view, already added as a subview of the keyboard
// view, or nil if the keyboard isn't showing.

// NOTE: this method returns a retained object. However, there is no need
// for the client code to release it, as that is done when one of the
// dismissal methods is invoked.

+ (WTFeedbackView*) newKeyboardFlexibleViewShowingProgressView: (BOOL) showing;

// ========================================================================= //

// Creates a keyboard-style feedback view, containing boh an activity indicator
// and a progress view (though only one can be shown at any given time), and
// the feedback label, vertically next to one another, with the entire
// feedback view centered within the keyboard view, if the keyboard is showing.
// Which indicator (activity or progress) is visible when the feedback view is
// first displayed is set by the specified boolean. The visible indicator can
// be switched on demand at any time, while the feedback view is showing.

// The label text is set to the specified string, though it can be changed
// on demand at any time while the feedback view is visible.

// Returns the feedback view, already added as a subview of the keyboard
// view, or nil if the keyboard isn't showing.

// NOTE: this method returns a retained object. However, there is no need
// for the client code to release it, as that is done when one of the
// dismissal methods is invoked.

+ (WTFeedbackView*) newKeyboardFlexibleViewShowingProgressView: (BOOL) showing
                                               withText: (NSString*) labelText;

// ========================================================================= //
                          #pragma mark accessor methods
// ========================================================================= //

// Returns the currently displayed feedback view, or nil if none is showing.

+ (WTFeedbackView*) currentFeedbackView;

// ========================================================================= //

// Returns the kind of the currently displayed feedback view, or
// WTFeedbackViewKindUnavailable if none is showing.

+ (WTFeedbackViewKind) kind;

// ========================================================================= //

// Returns the style of the currently displayed feedback view, or
// WTFeedbackViewStyleUnavailable if none is showing.

+ (WTFeedbackViewStyle) style;

// ========================================================================= //

// Returns the default text to be shown.
// The default implementation returns @"Loading...".

+ (NSString*) defaultFeedbackText;

// ========================================================================= //

// Returns the default localization string for the feedback text.
// The default implementation returns @"Default WTFeedbackView label text".

+ (NSString*) defaultFeedbackTextLocalizationString;

// ========================================================================= //

// Returns the feedback view's border view, the view that contains the
// activity indicator view and/or the progress view, and the feedback
// label.

// It's accessible so as to allow changing its appearance while the feedback
// view is showing. Any such changes should be executed in the main thread.

// The default implementation creates and returns a transparent view,
// with flexible margins.

// The bezel style has a semi-transparent (alpha = 0.5) black color, with
// a rounded rectangle (corner radius = 10.0).

// The keyboard style has a transparent border.

+ (UIView*) borderView;

// ========================================================================= //

// Returns the label displaying the feedback text.

// It's accessible so as to allow changing its appearance while the feedback
// view is showing. Any such changes should be executed in the main thread.

// The label's actual text should be changed using the class method +setText:,
// rather than directly, since +setText: guarantees thread-safety and also
// automatically repositions the subviews of the feedback view, as needed.

// The simple style has the following properties:

// backgroundColor  = [UIColor clearColor];
// font             = [UIFont systemFontOfSize: [UIFont systemFontSize]];
// textAlignment    = UITextAlignmentLeft;
// textColor        = [UIColor blackColor];
// shadowColor      = [UIColor whiteColor];
// shadowOffset     = CGSizeMake(0.0, 1.0);
// autoresizingMask = UIViewAutoresizingFlexibleRightMargin |
//                    UIViewAutoresizingFlexibleTopMargin;

// The bezel and keyboard styles have the following properties:

// backgroundColor  = [UIColor clearColor];
// font             = [UIFont boldSystemFontOfSize: [UIFont systemFontSize]];
// textAlignment    = UITextAlignmentCenter;
// textColor        = [UIColor whiteColor];
// autoresizingMask = UIViewAutoresizingFlexibleLeftMargin  |
//                    UIViewAutoresizingFlexibleRightMargin |
//                    UIViewAutoresizingFlexibleTopMargin;

+ (UILabel*) feedbackLabel;

// ========================================================================= //

// Returns the activity indicator view, or nil for feedback views of kind
// WTFeedbackViewKindProgress.

// It's accessible so as to allow changing its appearance while the feedback
// view is showing. Any such changes should be executed in the main thread.

// The default implementation returns nil.

// The simple style returns a small gray activity indicator.

// The bezel and keyboard styles return nil for the progress
// kind and a large white activity indicator for the activity
// and flexible kinds.

+ (UIActivityIndicatorView*) activityIndicatorView;

// ========================================================================= //

// Returns the progress view, or nil for feedback views of kind
// WTFeedbackViewKindActivity.

// It's accessible so as to allow changing its appearance while the feedback
// view is showing. Any such changes should be executed in the main thread.

// The default implementation returns nil. Only the
// progress and flexible kinds return a non-nil object.

+ (UIProgressView*) progressView;

// ========================================================================= //

// Returns whether or not the feedback view is currently showing its progress
// view, as opposed to its activity indicator view. Always returns NO for a
// feedback view of kind WTFeedbackViewKindActivity and always returns YES for
// a feedback view of kind WTFeedbackViewKindProgress.

// This method is thread-safe.

+ (BOOL) currentlyShowingProgressView;

// ========================================================================= //
                         #pragma mark mutator methods
// ========================================================================= //

// Sets the label text of the feedback view, automatically resizing and
// recentering the feedback view, if necessary.

// This method is guaranteed to execute its body in the main thread, so
// it's safe to invoke this method from any thread.

// This method should not be overriden.

+ (void) setText: (NSString*) text;

// ========================================================================= //

// For feedback views of kind WTFeedbackViewKindFlexible, hides the progress
// view and shows the activity indicator view. For feedback views of the other
// two kinds, it does nothing.

// This method is guaranteed to execute its body in the main thread, so
// it's safe to invoke this method from any thread.

// This method should not be overriden. Override the associated instance
// method instead, if necessary.

+ (void) switchToActivityIndicatorView;

// ========================================================================= //

// For feedback views of kind WTFeedbackViewKindFlexible, hides the activity
// indicator view and shows the progress view. For feedback views of the other
// two kinds, it does nothing.

// This method is guaranteed to execute its body in the main thread, so
// it's safe to invoke this method from any thread.

// This method should not be overriden. Override the associated instance
// method instead, if necessary.

+ (void) switchToProgressView;

// ========================================================================= //

// Does nothing for a feedback view of kind WTFeedbackViewKindActivity. For
// the other two kinds, updates the progress view, even if it's currently
// not showing.

// This method is guaranteed to execute its body in the main thread, so
// it's safe to invoke this method from any thread.

// This method should not be overriden. Override the associated instance
// method instead, if necessary.

+ (void) updateProgress: (CGFloat) progress;

// ========================================================================= //

// Immediately removes and releases the feedback view, without any animation.

// This method is guaranteed to execute its body in the main thread, so
// it's safe to invoke this method from any thread.

// This method should not be overriden.

+ (void) removeView;

// ========================================================================= //

// Removes and releases the feedback view, with or without an animation.
// The simple style does not support animating the feedback view removal,
// in which case the boolean argument is ignored, and +removeView is
// executed, instead.

// This method is guaranteed to execute its body in the main thread, so
// it's safe to invoke this method from any thread.

// This method should not be overriden. Override the associated instance
// method instead, if necessary.

+ (void) removeViewAnimating: (BOOL) animating;

// ========================================================================= //

@end

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ //
                                #pragma mark -
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ //

@interface WTFeedbackView ()

// ========================================================================= //
                          #pragma mark properties
// ========================================================================= //

@property (nonatomic, readwrite, assign) WTFeedbackViewKind  feedbackKind;
@property (nonatomic, readwrite, assign) WTFeedbackViewStyle feedbackStyle;

@property (nonatomic, readwrite, assign) CGSize labelTextSize;
@property (nonatomic, readwrite, assign) CGRect textBorderFrame;

@property (nonatomic, readwrite, retain) UIView* originalView;
@property (nonatomic, readwrite, retain) UIView*   borderView;

@property (nonatomic, readwrite, retain) UILabel* feedbackLabel;

@property (nonatomic, readwrite, retain)
    UIActivityIndicatorView* activityIndicatorView;

@property (nonatomic, readwrite, retain) UIProgressView* progressView;

@property (nonatomic, readwrite, assign) CGRect activityBorderFrame;
@property (nonatomic, readwrite, assign) CGRect progressBorderFrame;

// NOTE: atomic access.
@property (readwrite, assign) BOOL currentlyShowingProgressView;

// ========================================================================= //
                    #pragma mark memory management methods
// ========================================================================= //

// Designated initializer. Configures a feedback view of the specified kind
// and style, using the specified label text, and adds it as a subview of the
// specified view. If the feedback kind is anything other than
// WTFeedbackViewKindFlexible, the boolean argument is ignored (though the
// associated instance variable is correctly set anyway).

- (id) initWithKind: (WTFeedbackViewKind) kind
              style: (WTFeedbackViewStyle) style
            forView: (UIView*) addToView
           withText: (NSString*) labelText
showingProgressView: (BOOL) showing;

// ========================================================================= //
                       #pragma mark overridable methods
// ========================================================================= //

// Returns the view to which the feedback view is added.

// The default behavior is to return the same view passed as the argument.

// For the bezel style, if there is a keyboard displayed, the view
// is changed to the keyboard's superview.

// For the keyboard style, returns the same view (which should
// already be the keyboard).

- (UIView*) viewForView: (UIView*) view;

// ========================================================================= //

// Returns the frame to use for the feedback view.
// The default behavior is to return the superview's bounds.

// For the bezel style, if there is a keyboard displayed, the frame
// is changed to cover the keyboard too.

- (CGRect) enclosingFrame;

// ========================================================================= //

// Configures the feedback view's background. The default is to have
// a non-opaque background with an auto-resizing mask equal to
// UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight.

// For the bezel style, the background has a semi-transparent black color
// (alpha = 0.5).

// For the keyboard style, the background has a semi-transparent black
// color that's darker than that for the bezel style (alpha = 0.8).

- (void) setupBackground;

// ========================================================================= //

// Animates the feedback view into visibility.
// The default behavior is to do nothing.

// For the bezel style, fades in the background and zooms the
// bezel down from a large size.

// For the keyboard style, simply fades the feedback view in.

// This method is guaranteed to execute in the main thread.

- (void) animateShow;

// ========================================================================= //

// Animates the feedback view out of visibiltiy.
// The default behavior is to do nothing.

// For the bezel style, fades out the background and zooms the
// bezel down to half size.

// For the keyboard style, simply fades the feedback view out.

// This method is guaranteed to execute in the main thread.

- (void) animateRemove;

// ========================================================================= //

@end

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ //





// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ //
                                #pragma mark -
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ //

// A category on UIApplication.

@interface UIApplication (KeyboardView)

// Returns the keyboard view, if present, or nil.

//  Copyright Matt Gallagher 2009. All rights reserved.
//
//  Permission is given to use this source code file, free of charge, in any
//  project, commercial or otherwise, entirely at your risk, with the condition
//  that any redistribution (in part or whole) of source code must retain
//  this copyright and permission notice. Attribution in compiled projects is
//  appreciated but not required.

- (UIView*) keyboardView;

@end

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ //
